/*
	Copyright (c) 2008-2013, Anatoliy Belsky <ab@php.net>
	All rights reserved.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	- Redistributions of source code must retain the above copyright
	notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	notice, this list of conditions and the following disclaimer in the
	documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
	"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
	LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
	A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
	HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
	LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
	OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/* $Id: gender.c 335873 2015-02-03 17:21:42Z ab $ */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <php.h>
#include <php_ini.h>
#include <ext/standard/info.h>
#include <ext/pcre/php_pcre.h>
#include <zend_exceptions.h>

#include "gender_definitions.h"

#include "php_gender.h"

/* True global resources - no need for thread safety here */
static int le_gender;

/* class definitions */
static zend_class_entry *Gender_ce;

extern struct gc_struct gc_data[];

zend_object_handlers default_gender_handlers;

#include "nam_dict.c"

extern int initialize_gender (struct ze_gender_obj *zgo );
extern int get_gender (char first_name[], int compare_mode, int country, struct ze_gender_obj *zgo );
extern int determine_country (char *text, struct ze_gender_obj *zgo );

/* Off set length for gender_parse_dsn */
#define OFFSETS_LEN 21

int gender_connect_to_source(struct ze_gender_obj *zgo )
{/*{{{*/
	zgo->data_mode = DATA_MODE_ORIGINAL;

	if (zgo->f_names == NULL) {
		if (NULL != zgo->dsn) {
			zgo->f_names = php_stream_open_wrapper(zgo->dsn, "r",  STREAM_MUST_SEEK | USE_PATH, NULL);
			zgo->internal_dict = 0;
		} else {
			zgo->dsn = (char*)php_gender_name_dict_data;
			zgo->f_names = php_stream_open_wrapper(zgo->dsn, "r",  0, NULL);
			zgo->internal_dict = 1;
		}
		if (zgo->f_names == NULL) {
			zend_throw_exception_ex(Gender_ce, 0 , "Could not load data file");

			return INTERNAL_ERROR_GENDER;
		}
	}

	return 0;
}/*}}}*/

ZEND_DECLARE_MODULE_GLOBALS(gender);

/* {{{ gender_functions[]
 *
 * Every user visible function must have an entry in gender_functions[].
 */
zend_function_entry gender_functions[] = {
    {NULL, NULL, NULL}
};
/* }}} */

void gender_globals_dtor(zend_gender_globals *gender_globals )
{/*{{{*/
}/*}}}*/

void gender_globals_ctor(zend_gender_globals *gender_globals )
{/*{{{*/
}/*}}}*/

void
#if PHP_MAJOR_VERSION >= 7
php_gender_obj_destroy(zend_object *obj)
#else
php_gender_obj_destroy(void *obj )
#endif
{/*{{{*/
#if PHP_MAJOR_VERSION >= 7
	struct ze_gender_obj *zgo = php_gender_fetch_obj(obj);
#else
	struct ze_gender_obj *zgo = (struct ze_gender_obj *)obj;
#endif

	zend_object_std_dtor(&zgo->zo );

	if (!zgo->internal_dict) {
		efree(zgo->dsn);
	}

	if (zgo->f_names != NULL) {
		php_stream_close(zgo->f_names);
		zgo->f_names = NULL;
	}

	zgo->line_size = 0;
	zgo->record_count = 0L;
}/*}}}*/

#if PHP_MAJOR_VERSION >= 7
zend_object *
php_gender_obj_init(zend_class_entry *ze)
{
	struct ze_gender_obj *zgo;
	int i;

	zgo = (struct ze_gender_obj *)ecalloc(1, sizeof(struct ze_gender_obj));

	zend_object_std_init(&zgo->zo, ze);
	zgo->zo.handlers = &default_gender_handlers;

    zgo->internal_mode = GENDER_DEFAULT_TRACE_MODE;
    zgo->f_names = NULL;
    zgo->line_size = 0;
    zgo->record_count = 0L;

    zgo->data_mode = DATA_MODE_ORIGINAL;
    zgo->dsn = NULL;
	zgo->internal_dict = 1;

	/* NOTE gc_data has to be carried for each object,
	        otherwise the results might overlap in TS build */
	for (i=0; i < GENDER_GC_DATA_SIZE; i++) {
		zgo->gc_data[i].pos = gc_data[i].pos;
		zgo->gc_data[i].n = gc_data[i].n;
		zgo->gc_data[i].quality_of_statistics = gc_data[i].quality_of_statistics;
		zgo->gc_data[i].weight = gc_data[i].weight;
		zgo->gc_data[i].gc_country = gc_data[i].gc_country;
		zgo->gc_data[i].country_short = gc_data[i].country_short;
		zgo->gc_data[i].country_text = gc_data[i].country_text;
	}

	return &zgo->zo;
}
#else
zend_object_value
php_gender_obj_init(zend_class_entry *ze )
{/*{{{*/
	zend_object_value ret;
	struct ze_gender_obj *zgo;
#if PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION < 4
	zval *tmp;
#endif
	int i;

	zgo = (struct ze_gender_obj *)emalloc(sizeof(struct ze_gender_obj));
	memset(&zgo->zo, 0, sizeof(zend_object));

	zend_object_std_init(&zgo->zo, ze );
#if PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION < 4
	zend_hash_copy(zgo->zo.properties, &ze->default_properties, (copy_ctor_func_t) zval_add_ref,
				                    (void *) &tmp, sizeof(zval *));
#else
	object_properties_init(&zgo->zo, ze);
#endif

    zgo->internal_mode = GENDER_DEFAULT_TRACE_MODE;
    zgo->f_names = NULL;
    zgo->line_size = 0;
    zgo->record_count = 0L;

    zgo->data_mode = DATA_MODE_ORIGINAL;
    zgo->dsn = NULL;
	zgo->internal_dict = 1;

	/* NOTE gc_data has to be carried for each object,
	        otherwise the results might overlap in TS build */
	for (i=0; i < GENDER_GC_DATA_SIZE; i++) {
		zgo->gc_data[i].pos = gc_data[i].pos;
		zgo->gc_data[i].n = gc_data[i].n;
		zgo->gc_data[i].quality_of_statistics = gc_data[i].quality_of_statistics;
		zgo->gc_data[i].weight = gc_data[i].weight;
		zgo->gc_data[i].gc_country = gc_data[i].gc_country;
		zgo->gc_data[i].country_short = gc_data[i].country_short;
		zgo->gc_data[i].country_text = gc_data[i].country_text;
	}

    ret.handle = zend_objects_store_put(zgo, NULL,
				(zend_objects_free_object_storage_t) php_gender_obj_destroy,
				NULL );

	ret.handlers = &default_gender_handlers;

	return ret;
}/*}}}*/
#endif

/* {{{ client class methods[] */
zend_function_entry Gender_methods[] = {
	PHP_ME(Gender, __construct, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Gender, get, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Gender, connect, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Gender, trace, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Gender, country, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Gender, similarNames, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Gender, isNick, NULL, ZEND_ACC_PUBLIC)
    {NULL, NULL, NULL}
};
/* }}} */

ZEND_BEGIN_ARG_INFO_EX(Gender_get, 0, 0, 2)
    ZEND_ARG_INFO(0, firstname)
    ZEND_ARG_INFO(0, country)
ZEND_END_ARG_INFO()

/* {{{ gender_module_entry
 */
zend_module_entry gender_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
    STANDARD_MODULE_HEADER,
#endif
    "gender",
    gender_functions,
    PHP_MINIT(gender),
    PHP_MSHUTDOWN(gender),
    NULL,
    NULL,
    PHP_MINFO(gender),
#if ZEND_MODULE_API_NO >= 20010901
    PHP_GENDER_VERSION,
#endif
    STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_GENDER
ZEND_GET_MODULE(gender)
#endif


/* {{{ PHP_MINIT_FUNCTION
 */
PHP_MINIT_FUNCTION(gender)
{
    /* initalizing classes */
    zend_class_entry ce;

#ifdef ZTS
    ts_allocate_id(&gender_globals_id,
                   sizeof(zend_gender_globals),
                   (ts_allocate_ctor)gender_globals_ctor,
                   (ts_allocate_dtor)gender_globals_dtor);
#else
    gender_globals_ctor(&gender_globals );
#endif

	memcpy(&default_gender_handlers, zend_get_std_object_handlers(), sizeof(zend_object_handlers));
	default_gender_handlers.clone_obj = NULL;
#if PHP_MAJOR_VERSION >= 7
	default_gender_handlers.offset = XtOffsetOf(struct ze_gender_obj, zo);
	default_gender_handlers.free_obj = php_gender_obj_destroy;
#endif

#if PHP_MAJOR_VERSION >=7 || PHP_MAJOR_VERSION >= 5 && PHP_MINOR_VERSION >= 3
    INIT_CLASS_ENTRY(ce, "Gender\\Gender", Gender_methods);
#else
    INIT_CLASS_ENTRY(ce, "Gender", Gender_methods);
#endif
	ce.create_object = php_gender_obj_init;
    Gender_ce = zend_register_internal_class(&ce );

    /* name types list */
    zend_declare_class_constant_long(Gender_ce, "IS_FEMALE", sizeof("IS_FEMALE")-1, IS_FEMALE );
    zend_declare_class_constant_long(Gender_ce, "IS_MOSTLY_FEMALE", sizeof("IS_MOSTLY_FEMALE")-1, IS_MOSTLY_FEMALE );
    zend_declare_class_constant_long(Gender_ce, "IS_MALE", sizeof("IS_MALE")-1, IS_MALE );
    zend_declare_class_constant_long(Gender_ce, "IS_MOSTLY_MALE", sizeof("IS_MOSTLY_MALE")-1, IS_MOSTLY_MALE );
    zend_declare_class_constant_long(Gender_ce, "IS_UNISEX_NAME", sizeof("IS_UNISEX_NAME")-1, IS_UNISEX_NAME );
    zend_declare_class_constant_long(Gender_ce, "IS_A_COUPLE", sizeof("IS_A_COUPLE")-1, IS_A_COUPLE );
    zend_declare_class_constant_long(Gender_ce, "NAME_NOT_FOUND", sizeof("NAME_NOT_FOUND")-1, NAME_NOT_FOUND );
    zend_declare_class_constant_long(Gender_ce, "ERROR_IN_NAME", sizeof("ERROR_IN_NAME")-1, ERROR_IN_NAME );

    /* countries list */
    zend_declare_class_constant_long(Gender_ce, "ANY_COUNTRY", sizeof("ANY_COUNTRY")-1, GC_ANY_COUNTRY );

    zend_declare_class_constant_long(Gender_ce, "BRITAIN", sizeof("BRITAIN")-1, GC_BRITAIN );
    zend_declare_class_constant_long(Gender_ce, "IRELAND", sizeof("IRELAND")-1, GC_IRELAND );
    zend_declare_class_constant_long(Gender_ce, "USA", sizeof("USA")-1, GC_USA );

    zend_declare_class_constant_long(Gender_ce, "SPAIN", sizeof("SPAIN")-1, GC_SPAIN );
    zend_declare_class_constant_long(Gender_ce, "PORTUGAL", sizeof("PORTUGAL")-1, GC_PORTUGAL );
    zend_declare_class_constant_long(Gender_ce, "ITALY", sizeof("ITALY")-1, GC_ITALY );
    zend_declare_class_constant_long(Gender_ce, "MALTA", sizeof("MALTA")-1, GC_MALTA );

    zend_declare_class_constant_long(Gender_ce, "FRANCE", sizeof("FRANCE")-1, GC_FRANCE );
    zend_declare_class_constant_long(Gender_ce, "BELGIUM", sizeof("BELGIUM")-1, GC_BELGIUM );
    zend_declare_class_constant_long(Gender_ce, "LUXEMBOURG", sizeof("LUXEMBOURG")-1, GC_LUXEMBOURG );
    zend_declare_class_constant_long(Gender_ce, "NETHERLANDS", sizeof("NETHERLANDS")-1, GC_NETHERLANDS );

    zend_declare_class_constant_long(Gender_ce, "GERMANY", sizeof("GERMANY")-1, GC_GERMANY );
    zend_declare_class_constant_long(Gender_ce, "EAST_FRISIA", sizeof("EAST_FRISIA")-1, GC_EAST_FRISIA );
    zend_declare_class_constant_long(Gender_ce, "AUSTRIA", sizeof("AUSTRIA")-1, GC_AUSTRIA );
    zend_declare_class_constant_long(Gender_ce, "SWISS", sizeof("SWISS")-1, GC_SWISS );

    zend_declare_class_constant_long(Gender_ce, "ICELAND", sizeof("ICELAND")-1, GC_ICELAND );
    zend_declare_class_constant_long(Gender_ce, "DENMARK", sizeof("DENMARK")-1, GC_DENMARK );
    zend_declare_class_constant_long(Gender_ce, "NORWAY", sizeof("NORWAY")-1, GC_NORWAY );
    zend_declare_class_constant_long(Gender_ce, "SWEDEN", sizeof("SWEDEN")-1, GC_SWEDEN );
    zend_declare_class_constant_long(Gender_ce, "FINLAND", sizeof("FINLAND")-1, GC_FINLAND );
    zend_declare_class_constant_long(Gender_ce, "ESTONIA", sizeof("ESTONIA")-1, GC_ESTONIA );
    zend_declare_class_constant_long(Gender_ce, "LATVIA", sizeof("LATVIA")-1, GC_LATVIA );
    zend_declare_class_constant_long(Gender_ce, "LITHUANIA", sizeof("LITHUANIA")-1, GC_LITHUANIA );

    zend_declare_class_constant_long(Gender_ce, "POLAND", sizeof("POLAND")-1, GC_POLAND );
    zend_declare_class_constant_long(Gender_ce, "CZECH_REP", sizeof("CZECH_REP")-1, GC_CZECH_REP );
    zend_declare_class_constant_long(Gender_ce, "SLOVAKIA", sizeof("SLOVAKIA")-1, GC_SLOVAKIA );
    zend_declare_class_constant_long(Gender_ce, "HUNGARY", sizeof("HUNGARY")-1, GC_HUNGARY );
    zend_declare_class_constant_long(Gender_ce, "ROMANIA", sizeof("ROMANIA")-1, GC_ROMANIA );
    zend_declare_class_constant_long(Gender_ce, "BULGARIA", sizeof("BULGARIA")-1, GC_BULGARIA );

    zend_declare_class_constant_long(Gender_ce, "BOSNIA", sizeof("BOSNIA")-1, GC_BOSNIA );
    zend_declare_class_constant_long(Gender_ce, "CROATIA", sizeof("CROATIA")-1, GC_CROATIA );
    zend_declare_class_constant_long(Gender_ce, "KOSOVO", sizeof("KOSOVO")-1, GC_KOSOVO );
    zend_declare_class_constant_long(Gender_ce, "MACEDONIA", sizeof("MACEDONIA")-1, GC_MACEDONIA );
    zend_declare_class_constant_long(Gender_ce, "MONTENEGRO", sizeof("MONTENEGRO")-1, GC_MONTENEGRO );
    zend_declare_class_constant_long(Gender_ce, "SERBIA", sizeof("SERBIA")-1, GC_SERBIA );
    zend_declare_class_constant_long(Gender_ce, "SLOVENIA", sizeof("SLOVENIA")-1, GC_SLOVENIA );
    zend_declare_class_constant_long(Gender_ce, "ALBANIA", sizeof("ALBANIA")-1, GC_ALBANIA );
    zend_declare_class_constant_long(Gender_ce, "GREECE", sizeof("GREECE")-1, GC_GREECE );

    zend_declare_class_constant_long(Gender_ce, "RUSSIA", sizeof("RUSSIA")-1, GC_RUSSIA );
    zend_declare_class_constant_long(Gender_ce, "BELARUS", sizeof("BELARUS")-1, GC_BELARUS );
    zend_declare_class_constant_long(Gender_ce, "MOLDOVA", sizeof("MOLDOVA")-1, GC_MOLDOVA );
    zend_declare_class_constant_long(Gender_ce, "UKRAINE", sizeof("UKRAINE")-1, GC_UKRAINE );
    zend_declare_class_constant_long(Gender_ce, "ARMENIA", sizeof("ARMENIA")-1, GC_ARMENIA );
    zend_declare_class_constant_long(Gender_ce, "AZERBAIJAN", sizeof("AZERBAIJAN")-1, GC_AZERBAIJAN );
    zend_declare_class_constant_long(Gender_ce, "GEORGIA", sizeof("GEORGIA")-1, GC_GEORGIA );
    zend_declare_class_constant_long(Gender_ce, "KAZAKH_UZBEK", sizeof("KAZAKH_UZBEK")-1, GC_KAZAKH_UZBEK );

    zend_declare_class_constant_long(Gender_ce, "TURKEY", sizeof("TURKEY")-1, GC_TURKEY );
    zend_declare_class_constant_long(Gender_ce, "ARABIA", sizeof("ARABIA")-1, GC_ARABIA );
    zend_declare_class_constant_long(Gender_ce, "ISRAEL", sizeof("ISRAEL")-1, GC_ISRAEL );
    zend_declare_class_constant_long(Gender_ce, "CHINA", sizeof("CHINA")-1, GC_CHINA );
    zend_declare_class_constant_long(Gender_ce, "INDIA", sizeof("INDIA")-1, GC_INDIA );
    zend_declare_class_constant_long(Gender_ce, "JAPAN", sizeof("JAPAN")-1, GC_JAPAN );
    zend_declare_class_constant_long(Gender_ce, "KOREA", sizeof("KOREA")-1, GC_KOREA );
    zend_declare_class_constant_long(Gender_ce, "VIETNAM", sizeof("VIETNAM")-1, GC_VIETNAM );

    return SUCCESS;
}
/* }}} */

/* {{{ PHP_MSHUTDOWN_FUNCTION
 */
PHP_MSHUTDOWN_FUNCTION(gender)
{

#ifdef ZTS
    ts_free_id(gender_globals_id);
#else
    gender_globals_dtor(&gender_globals );
#endif

    return SUCCESS;
}
/* }}} */

/* {{{ PHP_MINFO_FUNCTION
 */
PHP_MINFO_FUNCTION(gender)
{
    php_info_print_table_start();
    php_info_print_table_header(2, "gender support", "enabled");
    php_info_print_table_header(2, "Version ", PHP_GENDER_VERSION);
    php_info_print_table_header(2, "Gender API ", GENDER_VERSION);
    php_info_print_table_end();
}
/* }}} */


/* {{{ proto Gender Gender::Gender([string $dsn]) construct Gender optionally connecting to data source at the same time */
PHP_METHOD(Gender, __construct)
{
	struct ze_gender_obj *zgo;
    int i;
    char *dsn = NULL;
#if PHP_MAJOR_VERSION >= 7
	size_t dsn_len = 0;
#else
	int dsn_len = 0;
#endif

    if(zend_parse_parameters(ZEND_NUM_ARGS() , "|s", &dsn, &dsn_len) == FAILURE) {
            return;
    }

#if PHP_MAJOR_VERSION >= 7
	zgo = php_gender_fetch_obj(Z_OBJ_P(getThis()));
#else
	zgo = (struct ze_gender_obj *) zend_object_store_get_object(getThis() );
#endif

    if(dsn_len > 0) {
        zgo->dsn = estrdup(dsn);
        if(gender_connect_to_source(zgo ) == INTERNAL_ERROR_GENDER) {
            RETURN_NULL();
        }
    }

    if (! (zgo->internal_mode & GENDER_INITIALIZED)) {
        i = initialize_gender(zgo );

        if (i < 0  ||  ! (zgo->internal_mode & GENDER_INITIALIZED)) {
            zend_throw_exception(Gender_ce, "Initialization failed", 0 );
            RETURN_NULL();
        }
    }
}
/* }}} */


/* {{{ proto integer Gender::get(string $name, string $country) get gender for the name in the country */
PHP_METHOD(Gender, get)
{
	struct ze_gender_obj *zgo;
    char *firstname;
#if PHP_MAJOR_VERSION >= 7
    size_t firstname_len;
#else
    int firstname_len;
#endif
    zval *country = NULL;

    if (zend_parse_parameters(ZEND_NUM_ARGS() , "s|z", &firstname, &firstname_len, &country) == FAILURE) {
            return;
    }

#if PHP_MAJOR_VERSION >= 7
	zgo = php_gender_fetch_obj(Z_OBJ_P(getThis()));
#else
	zgo = (struct ze_gender_obj *) zend_object_store_get_object(getThis() );
#endif

    if(firstname_len) {
		int mode = 0;
		zend_long gc_country = GC_ANY_COUNTRY;

        if(country) {
            if(Z_TYPE_P(country) == IS_LONG) {
                gc_country = Z_LVAL_P(country);
            } else if(Z_TYPE_P(country) == IS_STRING) {
                gc_country = determine_country(Z_STRVAL_P(country), zgo );
            }
			mode = IGNORE_OTHER_COUNTRIES;
        }

        ZVAL_LONG(return_value, get_gender(firstname, mode, gc_country, zgo ));

        return;
    }

    RETURN_FALSE;
}
/* }}} */


/* {{{ proto boolean Gender::connect(string $dsn) Open a data source, will support several sources*/
PHP_METHOD(Gender, connect)
{
	struct ze_gender_obj *zgo;
    char *dsn;
#if PHP_MAJOR_VERSION >= 7
    size_t dsn_len;
#else
    int dsn_len;
#endif

    if (zend_parse_parameters(ZEND_NUM_ARGS() , "s", &dsn, &dsn_len) == FAILURE) {
            RETURN_FALSE;
    }

#if PHP_MAJOR_VERSION >= 7
	zgo = php_gender_fetch_obj(Z_OBJ_P(getThis()));
#else
	zgo = (struct ze_gender_obj *) zend_object_store_get_object(getThis() );
#endif

    if(dsn_len) {
		if (zgo->dsn && !zgo->internal_dict) {
			efree(zgo->dsn);
			zgo->dsn = estrdup(dsn);
		}

        if(gender_connect_to_source(zgo ) != INTERNAL_ERROR_GENDER) {
            RETURN_TRUE;
        }
    }

    RETURN_FALSE;

}
/* }}} */


/* {{{ proto boolean Gender::trace(void) allow trace mode */
PHP_METHOD(Gender, trace)
{
	struct ze_gender_obj *zgo;

#if PHP_MAJOR_VERSION >= 7
	zgo = php_gender_fetch_obj(Z_OBJ_P(getThis()));
#else
	zgo = (struct ze_gender_obj *) zend_object_store_get_object(getThis() );
#endif

    zgo->internal_mode = TRACE_GENDER;
}
/* }}} */


/* {{{ proto string Gender::country(integer id) Get string country representation from a gender country class constant. */
PHP_METHOD(Gender, country)
{
	zend_long country;
	int i;
	struct ze_gender_obj *zgo;

    if (zend_parse_parameters(ZEND_NUM_ARGS() , "l", &country) == FAILURE) {
		RETURN_FALSE;
	}

#if PHP_MAJOR_VERSION >= 7
	zgo = php_gender_fetch_obj(Z_OBJ_P(getThis()));
#else
	zgo = (struct ze_gender_obj *) zend_object_store_get_object(getThis() );
#endif

	array_init(return_value);
	for (i=0; zgo->gc_data[i].country_text != NULL; i++) {
		if(zgo->gc_data[i].gc_country == (int)country) {
#if PHP_MAJOR_VERSION >= 7
			add_assoc_stringl(return_value, "country_short", zgo->gc_data[i].country_short, strlen(zgo->gc_data[i].country_short));
			add_assoc_stringl(return_value, "country", zgo->gc_data[i].country_text, strlen(zgo->gc_data[i].country_text));
#else
			add_assoc_stringl(return_value, "country_short", zgo->gc_data[i].country_short, strlen(zgo->gc_data[i].country_short), 1);
			add_assoc_stringl(return_value, "country", zgo->gc_data[i].country_text, strlen(zgo->gc_data[i].country_text), 1);
#endif
			break;
		}
	}
}
/* }}} */

/*{{{ proto string Gender::similarNames(string name[, integer country]) Find similar name*/
PHP_METHOD(Gender, similarNames)
{
	zend_long country = GC_ANY_COUNTRY;
#if PHP_MAJOR_VERSION >= 7
	size_t name_len = 0;
#else
	int name_len = 0;
#endif
	char *name, *buf, *pch0, *pch1;
	struct ze_gender_obj *zgo;

	if (zend_parse_parameters(ZEND_NUM_ARGS() , "s|l", &name, &name_len, &country) == FAILURE) {
		RETURN_FALSE;
	}

#if PHP_MAJOR_VERSION >= 7
	zgo = php_gender_fetch_obj(Z_OBJ_P(getThis()));
#else
	zgo = (struct ze_gender_obj *) zend_object_store_get_object(getThis() );
#endif

	/* XXX possible overflow */
	buf = (char *) emalloc(sizeof(char)*2048);
	array_init(return_value);

	find_similar_name(name, (int)country, buf, 2048, zgo );

	if (strlen(buf) < 1) {
		efree(buf);
		return;
	}

	pch0 = buf;
	pch1 = strchr(buf, ';');

	if (NULL == pch1) {
#if PHP_MAJOR_VERSION >= 7
		add_next_index_string(return_value, buf);
#else
		add_next_index_string(return_value, buf, 1);
#endif
	} else {
		char *one;

		while(NULL != pch1) {
#if PHP_MAJOR_VERSION >= 7
			ptrdiff_t one_len = pch1 - pch0;
#else
			int one_len = pch1 - pch0;
#endif

			one = estrndup(pch0, one_len);
			one[one_len] = '\0';
#if PHP_MAJOR_VERSION >= 7
			add_next_index_string(return_value, one);
			efree(one);
#else
			add_next_index_string(return_value, one, 0);
#endif

			pch0 = pch1 + 2;
			pch1 = strchr(pch0, ';');
		}
#if PHP_MAJOR_VERSION >= 7
		add_next_index_string(return_value, pch0);
#else
		add_next_index_string(return_value, pch0, 1);
#endif
	}

	efree(buf);
}
/*}}}*/

/*{{{ proto string Gender::isNick(string name0, string name1[, int country]) Check if name0 is a nick of name1 */
PHP_METHOD(Gender, isNick)
{
	zend_long country = GC_ANY_COUNTRY;
	char *name0, *name1;
#if PHP_MAJOR_VERSION >= 7
	size_t name0_len, name1_len;
#else
	int name0_len, name1_len;
#endif
	int check;
	struct ze_gender_obj *zgo;

	if (zend_parse_parameters(ZEND_NUM_ARGS() , "ss|l", &name0, &name0_len, &name1, &name1_len, &country) == FAILURE) {
		RETURN_FALSE;
	}

#if PHP_MAJOR_VERSION >= 7
	zgo = php_gender_fetch_obj(Z_OBJ_P(getThis()));
#else
	zgo = (struct ze_gender_obj *) zend_object_store_get_object(getThis() );
#endif

	check = check_nickname(name0, name1, 0, (int)country, zgo );

	RETURN_BOOL(EQUIVALENT_NAMES == check);
}
/*}}}*/

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
