/*
	Copyright (c) 2008-2013, Anatoliy Belsky <ab@php.net>
	All rights reserved.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	- Redistributions of source code must retain the above copyright
	notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	notice, this list of conditions and the following disclaimer in the
	documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
	"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
	LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
	A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
	HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
	LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
	OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/* $Id: php_gender.h 335839 2015-01-29 15:21:22Z ab $ */

#ifndef PHP_GENDER_H
#define PHP_GENDER_H

extern zend_module_entry gender_module_entry;
#define phpext_gender_ptr &gender_module_entry

#ifdef PHP_WIN32
#define PHP_GENDER_API __declspec(dllexport)
#else
#define PHP_GENDER_API
#endif

#ifdef ZTS
#include "TSRM.h"
#endif

#define PHP_GENDER_VERSION "1.1.0"

PHP_MINIT_FUNCTION(gender);
PHP_MSHUTDOWN_FUNCTION(gender);
PHP_MINFO_FUNCTION(gender);

PHP_METHOD(Gender, __construct);
PHP_METHOD(Gender, get);
PHP_METHOD(Gender, connect);
PHP_METHOD(Gender, trace);
PHP_METHOD(Gender, country);
PHP_METHOD(Gender, similarNames);
PHP_METHOD(Gender, isNick);

#ifdef ZTS
#define GENDER_G(v) TSRMG(gender_globals_id, zend_gender_globals *, v)
#else
#define GENDER_G(v) (gender_globals.v)
#endif

/* data modes list */
#define DATA_MODE_ORIGINAL 0
#define DATA_MODE_SQLITE 1
#define DATA_MODE_MYSQL 2
#define DATA_MODE_POSTGRES 3

ZEND_BEGIN_MODULE_GLOBALS(gender)
	int dummy;
ZEND_END_MODULE_GLOBALS(gender)

ZEND_EXTERN_MODULE_GLOBALS(gender)

#define GENDER_GC_DATA_SIZE 56
struct ze_gender_obj {
#if PHP_MAJOR_VERSION < 7
	zend_object zo;
#endif
    int  internal_mode;
    php_stream *f_names;
    int  line_size;
    long record_count;
    long cache_p [CACHE_LOOKBACK_COUNT];
    char cache_text [CACHE_LOOKBACK_COUNT] [MAX_LINE_SIZE+1];

    char up_and_conv[HASH_COUNT];
    char sortchar [HASH_COUNT];
    char sortchar2[HASH_COUNT];
    char upperchar[HASH_COUNT];

    int ph_hash_group_begin[17];
    int ph_hash_group_end[17];
    int ph_rules_hash_del_ins [HASH_COUNT];
    int ph_rules_hash_replace [HASH_COUNT];

    char trace_buffer[2001];
    char input_buffer_1 [101];
    char input_buffer_2 [101];
    char output_buffer [2002];

    int data_mode;
    char *dsn;
	int internal_dict;
	struct gc_struct gc_data[GENDER_GC_DATA_SIZE];
#if PHP_MAJOR_VERSION >= 7
	zend_object zo;
#endif
};

#if PHP_MAJOR_VERSION >= 7
static zend_always_inline struct ze_gender_obj *
php_gender_fetch_obj(zend_object *obj)
{/*{{{*/
	return (struct ze_gender_obj *)((char *)obj - XtOffsetOf(struct ze_gender_obj, zo));
}/*}}}*/
#endif

#if PHP_MAJOR_VERSION < 7
typedef long zend_long;
#endif

#endif	/* PHP_GENDER_H */


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
